#include <stdint.h>

#include <libopencm3/stm32/gpio.h>
#include <libopencm3/stm32/spi.h>
#include "FreeRTOS.h"
#include "task.h"

#include "ssd1306.h"


static void oled_command(struct ssd1306 *dev, uint8_t byte) {
	gpio_clear(dev->port,dev->dc);
	spi_enable(dev->spi);
	spi_xfer(dev->spi,byte);
	spi_disable(dev->spi);
}

static void oled_command2(struct ssd1306 *dev, uint8_t byte,uint8_t byte2) {
	gpio_clear(dev->port,dev->dc);
	spi_enable(dev->spi);
	spi_xfer(dev->spi,byte);
	spi_xfer(dev->spi,byte2);
	spi_disable(dev->spi);
}

static void oled_data(struct ssd1306 *dev, uint8_t byte) {
	gpio_set(dev->port,dev->dc);
	spi_enable(dev->spi);
	spi_xfer(dev->spi,byte);
	spi_disable(dev->spi);
}

void oled_reset(struct ssd1306 *dev) {
	gpio_clear(dev->port,dev->rst);
    // TODO: that's ugly because it depends on 1) freertos 2) being inside a task
	vTaskDelay(1);
	gpio_set(dev->port,dev->rst);
}

void oled_init(struct ssd1306 *dev) {
	static uint8_t cmds[] = {
		0xAE, 0x00, 0x10, 0x40, 0x81, 0xCF, 0xA1, 0xA6,
		0xA8, 0x3F, 0xD3, 0x00, 0xD5, 0x80, 0xD9, 0xF1,
		0xDA, 0x12, 0xDB, 0x40, 0x8D, 0x14, 0xAF, 0xFF };

	oled_reset(dev);
	for ( unsigned ux=0; cmds[ux] != 0xFF; ++ux )
		oled_command(dev, cmds[ux]);
}

void oled_setup(struct ssd1306 *dev) {
    gpio_set_mode(dev->port, GPIO_MODE_OUTPUT_2_MHZ,
            GPIO_CNF_OUTPUT_PUSHPULL, dev->dc | dev->rst);
    gpio_clear(dev->port, dev->rst);
    gpio_set_mode(dev->port, GPIO_MODE_OUTPUT_50_MHZ,
            GPIO_CNF_OUTPUT_ALTFN_PUSHPULL,
            dev->clock | dev->mosi | dev->cs);
    spi_reset(dev->spi);
    spi_init_master( dev->spi,
            SPI_CR1_BAUDRATE_FPCLK_DIV_256,
            SPI_CR1_CPOL_CLK_TO_0_WHEN_IDLE,
            SPI_CR1_CPHA_CLK_TRANSITION_1,
            SPI_CR1_DFF_8BIT,
            SPI_CR1_MSBFIRST
            );
    spi_disable_software_slave_management(dev->spi);
    spi_enable_ss_output(dev->spi);
}

// buf must be exactly 128*64/8 bytes
void oled_draw(struct ssd1306 *dev, uint8_t *buf) {
	oled_command2(dev, 0x20,0x02);// Page mode
	oled_command(dev, 0x40);
	oled_command2(dev, 0xD3,0x00);
	for ( uint8_t px=0; px<8; ++px ) {
		oled_command(dev, 0xB0|px);
		oled_command(dev, 0x00); // Lo col
		oled_command(dev, 0x10); // Hi col
		for ( unsigned bx=0; bx<128; ++bx ) {
			oled_data(dev, *buf++);
        }
	}
}


